@@ -1,13 +1,13 @@
 """Functions for selecting residues and atoms from PDB structure."""
 
-from itertools import chain
+import itertools
 
 from Bio.PDB.Atom import Atom
 from Bio.PDB.Chain import Chain
 from Bio.PDB.Residue import Residue
 from Bio.PDB.Structure import Structure
 
-from smoltools.pdbtools.exceptions import ChainNotFound
+from smoltools.pdbtools.exceptions import ChainNotFound, NoResiduesFound, NoAtomsFound
 
 
 def get_chain(structure: Structure, model: int, chain: str) -> Chain:
@@ -44,14 +44,25 @@ def get_residues(chain: Chain, residue_filter: set[str] = None) -> list[Residue]
     list[Residue]: List of PDB residue objects in the given entity that meet the
         residue filter.
     """
-    residues = [residue for residue in chain.get_residues()]
     if residue_filter is None:
-        return [residue for residue in residues if residue.get_id()[0] == ' ']
-    else:
-        return [
-            residue for residue in residues if residue.get_resname() in residue_filter
+        residues = [
+            residue for residue in chain.get_residues() if residue.get_id()[0] == ' '
         ]
-    # TODO: error handling for empty residue list?
+    else:
+        residues = [
+            residue
+            for residue in chain.get_residues()
+            if residue.get_resname() in residue_filter
+        ]
+
+    return _validate_residues(residues)
+
+
+def _validate_residues(residues: list[Residue]) -> list[Residue]:
+    if not residues:
+        raise NoResiduesFound
+    else:
+        return residues
 
 
 def get_alpha_carbons(residues: list[Residue]) -> list[Atom]:
@@ -66,16 +77,15 @@ def get_alpha_carbons(residues: list[Residue]) -> list[Atom]:
     list[Atom]: list of alpha carbons as PDB atom objects.
     """
 
-    def _get_alpha_carbon(residue: Residue) -> Atom:
-        alpha_carbons = [atom for atom in residue.get_atoms() if atom.get_id() == 'CA']
-        if not alpha_carbons:
-            print(f'{residue} has no alpha carbon')
+    def _get_atoms(residue) -> list[Atom]:
+        return [atom for atom in residue.get_atoms() if atom.get_id() == 'CA']
 
-        return alpha_carbons
+    atoms = _flatten_list([_get_atoms(residue) for residue in residues])
 
-    alpha_carbons = [_get_alpha_carbon(residue) for residue in residues]
-    return list(chain(*alpha_carbons))
-    # TODO: error handling for empty residue list?
+    if not atoms:
+        raise NoAtomsFound
+    else:
+        return atoms
 
 
 def get_carbons(
@@ -86,16 +96,26 @@ def get_carbons(
     amino acid.
     """
 
-    def _get_atoms(residue: Residue, atom_names: list[str]):
-        return [atom for atom in residue.get_atoms() if atom.get_name() in atom_names]
+    def _get_atoms(residue: Residue):
+        atom_filter = atom_select[residue.get_resname()]
+        return [atom for atom in residue.get_atoms() if atom.get_name() in atom_filter]
 
-    atoms = [
-        _get_atoms(residue, atom_select[residue.get_resname()]) for residue in residues
-    ]
-    return list(chain(*atoms))
-    # TODO: error handling for empty residue list?
+    atoms = _flatten_list([_get_atoms(residue) for residue in residues])
+    return _validate_atoms(atoms)
 
 
 def filter_by_b_factor(atoms: list[Atom], cutoff) -> list[Atom]:
     """Returns a list of atoms with a b factor that meets the provided cutoff."""
-    return [atom for atom in atoms if atom.get_bfactor() > cutoff]
+    atoms = [atom for atom in atoms if atom.get_bfactor() > cutoff]
+    return _validate_atoms(atoms)
+
+
+def _validate_atoms(atoms=list[Atom]) -> list[Atom]:
+    if not atoms:
+        raise NoAtomsFound
+    else:
+        return atoms
+
+
+def _flatten_list(nested_list: list[list]) -> list:
+    return list(itertools.chain(*nested_list))
